package com.daim.flutter_apm

import androidx.annotation.NonNull
import com.moonmiles.apm.utils.deeplink.APMDeeplinkListener
import com.moonmiles.apm.utils.deeplink.APMDeeplinkUtils
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel

/** APMDeeplinkUtilsModule */
class APMDeeplinkUtilsModule : FlutterPlugin, EventChannel.StreamHandler, APMDeeplinkListener {
    private lateinit var eventChannel: EventChannel
    private var deeplinkUtilsListener: Any? = null
    private var deeplinkUtilsEventSink: EventChannel.EventSink? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        eventChannel = EventChannel(flutterPluginBinding.binaryMessenger, "apm_deeplink_utils_module_events")
        eventChannel.setStreamHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        eventChannel.setStreamHandler(null)
    }

    //************************************************************************* StreamHandler

    override fun onListen(listener: Any?, eventSink: EventChannel.EventSink?) {
        APMDeeplinkUtils.getInstance().addListener(this)
        this.deeplinkUtilsListener = listener
        this.deeplinkUtilsEventSink = eventSink
    }

    override fun onCancel(listener: Any?) {
        APMDeeplinkUtils.getInstance().removeListener(this)
        this.deeplinkUtilsListener = null
        this.deeplinkUtilsEventSink = null
    }

    //******************************************************************* APMDeeplinkListener

    override fun apmDeeplinkOnboardingButtonOkClicked() {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmDeeplinkOnboardingButtonOkClicked"
        deeplinkUtilsEventSink?.success(result)
    }

    override fun apmDeeplinkOnboardingButtonLaterClicked() {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmDeeplinkOnboardingButtonLaterClicked"
        deeplinkUtilsEventSink?.success(result)
    }

    override fun apmDeeplinkOnboardingButtonNeverClicked() {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmDeeplinkOnboardingButtonNeverClicked"
        deeplinkUtilsEventSink?.success(result)
    }

    override fun apmDeeplinkActionButtonGoClicked(url: String?, action: String?, params: java.util.HashMap<*, *>?) {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmDeeplinkActionButtonGoClicked"
        if(url != null) result["url"] = url
        if(action != null) result["action"] = action
        if(params != null) result["params"] = params
        deeplinkUtilsEventSink?.success(result)
    }

    override fun apmDeeplinkClicked(url: String?): Boolean {
        val result: HashMap<String, Any> = HashMap()
        result["method"] = "apmDeeplinkClicked"
        if(url != null) result["url"] = url
        deeplinkUtilsEventSink?.success(result)
        return false
    }
}
