package com.daim.flutter_apm

import androidx.annotation.NonNull
import com.moonmiles.apm.sdk.service.APMUIServicesUser
import com.moonmiles.apmservices.model.APMUser
import com.moonmiles.apmservices.net.APMException
import com.moonmiles.apmservices.sdk.user.APMUserConnectListener
import com.moonmiles.apmservices.sdk.user.APMUserDeleteListener
import com.moonmiles.apmservices.sdk.user.APMUserLogoutListener
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.util.*
import kotlin.collections.HashMap

/** APMUIServicesUserModule */
class APMUIServicesUserModule : FlutterPlugin, MethodCallHandler {
    private lateinit var channel: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "apm_ui_services_user_module")
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        val arguments: ArrayList<*>
        if(call.arguments != null) {
            arguments = call.arguments as ArrayList<*>
        }
        else {
            arguments = ArrayList<Any>()
        }

        if (call.method == "userClientConnectWithJSON") {
            userClientConnectWithJSON(arguments, result)
        }
        else if (call.method == "userClientConnect") {
            userClientConnect(arguments, result)
        }
        else if (call.method == "userLogout") {
            userLogout(arguments, result)
        }
        else if (call.method == "userDelete") {
            userDelete(arguments, result)
        }
        else {
            result.notImplemented()
        }
    }

    private fun userClientConnectWithJSON(arguments: ArrayList<*>, result: Result) {
        APMUIServicesUser.userClientConnect(arguments[0] as HashMap<*, *>?, object : APMUserConnectListener {
            override fun userConnectSuccess(user: APMUser) {
                APMFlutterUtils.resolveResult(user, result)
            }

            override fun failure(e: APMException) {
                APMFlutterUtils.rejectResult(e, result)
            }
        })
    }

    private fun userClientConnect(arguments: ArrayList<*>, result: Result) {
        val firstname: String? = arguments[0] as? String
        val lastname: String? = arguments[1] as? String
        val email: String? = arguments[2] as? String
        val facebookId: String? = arguments[3] as? String
        val partnerClientId: String? = arguments[4] as? String
        val oldClientId: String? = arguments[5] as? String
        val segments: HashMap<*, *>? = arguments[6] as? HashMap<*, *>
        val optIn: Int? = arguments[7] as? Int

        APMUIServicesUser.userClientConnect(firstname, lastname, email, facebookId, partnerClientId, oldClientId, segments, optIn, object : APMUserConnectListener {
            override fun userConnectSuccess(user: APMUser?) {
                APMFlutterUtils.resolveResult(user, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }

    private fun userLogout(arguments: ArrayList<*>, result: Result) {
        APMUIServicesUser.userLogout(object : APMUserLogoutListener {
            override fun userLogoutSuccess() {
                APMFlutterUtils.resolveResult(null, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }

    private fun userDelete(arguments: ArrayList<*>, result: Result) {
        APMUIServicesUser.userDelete(object : APMUserDeleteListener {
            override fun userDeleteSuccess() {
                APMFlutterUtils.resolveResult(null, result)
            }

            override fun failure(exception: APMException?) {
                APMFlutterUtils.rejectResult(exception, result)
            }
        })
    }
}
