//
//  APMFlutterUtils.swift
//  Runner
//
//  Created by Vincent Ducastel on 21/07/2021.
//

import Foundation
import APM

class APMFlutterUtils {
    static func errorFromCode(code: Int, exception: NSException) -> NSError {
        var desc: String = String.init(format: "[%@]", exception.name.rawValue)
        if(exception.reason != nil) {
            desc.append(" ")
            desc.append(exception.reason!)
        }
        let errorInfo: [String : Any] = [NSLocalizedDescriptionKey:desc]
        let error: NSError = NSError.init(domain: "", code: code, userInfo: errorInfo)
        return error
    }
    
    //Resolve
    //********************//
    
    static func resolveResult(object: Any?, result: FlutterResult) {
        if(object != nil) {
            if(object is NSArray) {
                resolveResultWithArray(array: object as? NSArray, result: result)
            }
            else if(object is APMModel) {
                let model : APMModel = object as! APMModel
                let dico: NSMutableDictionary = model.toDictionary()
                removeNotCompatibleObjects(dictionary: dico)
                result(dico)
            }
            else {
                result(object)
            }
        }
        else {
            result(nil)
        }
    }
    
    static func resolveResultWithArray(array: NSArray?, result: FlutterResult) {
        if(array != nil && array!.count > 0 && array!.firstObject is APMModel) {
            let arrayDictionary: NSMutableArray = APMReactUtils.toArrayOfDictionary(array as? [Any])
            removeNotCompatibleObjects(array: arrayDictionary)
            result(arrayDictionary)
        }
        else {
            result(array)
        }
    }
    
    //Reject
    //********************//
    static func rejectResult(object: Any?, result: FlutterResult) {
        if(object != nil) {
            if(object is NSError) {
                rejectResultWithError(error: object as! NSError, result: result)
            }
            else if(object is NSException) {
                rejectResultWithException(exception: object as! NSException, result: result)
            }
        }
    }

    static func rejectResultWithException(exception: NSException, result: FlutterResult) {
        let error: NSError = errorFromCode(code: 500, exception: exception)
        rejectResultWithError(error: error, result: result)
    }

    static func rejectResultWithError(error: NSError, result: FlutterResult) {
        result(FlutterError.init(code: String.init(format: "%li", error.code), message: error.localizedDescription, details: error.localizedDescription))
    }
    
    //Others
    //********************//
    static func removeNotCompatibleObjects(array: NSMutableArray) {
        for dictionary in array {
            let dictionary: NSMutableDictionary = dictionary as! NSMutableDictionary
            removeNotCompatibleObjects(dictionary: dictionary)
        }
    }
    
    static func removeNotCompatibleObjects(dictionary: NSMutableDictionary) {
        //Remove all objects APMImage
        for objectDico in dictionary {
            if(objectDico.value is APMImage) {
                dictionary.removeObject(forKey: objectDico.key)
            }
        }
    }
}
