import Flutter
import UIKit
import APM

public class APMDeeplinkUtilsModule: NSObject, FlutterPlugin, FlutterStreamHandler, APMDeeplinkListener {
    private var deeplinkUtilsEventSink: FlutterEventSink?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let instance = APMDeeplinkUtilsModule()
        let eventChannel = FlutterEventChannel(name: "apm_deeplink_utils_module_events", binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler(instance)
    }

    //************************************************************************* StreamHandler
    
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        APMDeeplinkUtils.sharedInstance()?.add(self)
        self.deeplinkUtilsEventSink = events
        return nil
    }
    
    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        APMDeeplinkUtils.sharedInstance()?.remove(self)
        self.deeplinkUtilsEventSink = nil
        return nil
    }
    
    //******************************************************************* APMDeeplinkListener
    
    public func apmDeeplinkOnboardingButtonOkClicked() {
        var result: [String: Any] = [:]
        result["method"] = "apmDeeplinkOnboardingButtonOkClicked"
        self.deeplinkUtilsEventSink?(result)
    }
    
    public func apmDeeplinkOnboardingButtonLaterClicked() {
        var result: [String: Any] = [:]
        result["method"] = "apmDeeplinkOnboardingButtonLaterClicked"
        self.deeplinkUtilsEventSink?(result)
    }
    
    public func apmDeeplinkOnboardingButtonNeverClicked() {
        var result: [String: Any] = [:]
        result["method"] = "apmDeeplinkOnboardingButtonNeverClicked"
        self.deeplinkUtilsEventSink?(result)
    }
    
    public func apmDeeplinkActionButtonGoClicked(_ url: String!, action: String!, params: NSMutableDictionary!) {
        var result: [String: Any] = [:]
        result["method"] = "apmDeeplinkActionButtonGoClicked"
        if(url != nil) { result["url"] = url }
        if(action != nil) { result["action"] = action }
        if(params != nil) { result["params"] = params }
        self.deeplinkUtilsEventSink?(result)
    }
    
    public func apmDeeplinkClicked(_ url: String!) -> Bool {
        var result: [String: Any] = [:]
        result["method"] = "apmDeeplinkClicked"
        if(url != nil) { result["url"] = url }
        self.deeplinkUtilsEventSink?(result)
        return false
    }
}
