
import 'dart:async';
import 'package:flutter/services.dart';

class APMModule {
  static const MethodChannel _channel = const MethodChannel('apm_module');
  static const EventChannel _servicesUserEventChannel = const EventChannel('apm_module_services_user_event');
  static StreamSubscription? servicesUserSubscription;
  static const EventChannel _onBadgeClickEventChannel = const EventChannel('apm_module_on_badge_click_event');
  static StreamSubscription? onBadgeClickSubscription;

  static Future initAppWithPartnerID(String partnerID, String partnerSecret) async {
    await _channel.invokeMethod('initAppWithPartnerID', [partnerID, partnerSecret]);
  }

  static Future initAppWithAppID(String partnerID, String partnerSecret, String appId) async {
    await _channel.invokeMethod('initAppWithAppID', [partnerID, partnerSecret, appId]);
  }

  static Future refreshSDK(bool force) async {
    await _channel.invokeMethod('refreshSDK', [force]);
  }

  static Future setServicesUserListener(void onData(dynamic event)?) async {
    if(onData != null) {
      if(servicesUserSubscription != null) {
        servicesUserSubscription!.cancel();
        servicesUserSubscription = null;
      }
      servicesUserSubscription = _servicesUserEventChannel.receiveBroadcastStream().listen(onData);  
    }
    else {
      if(servicesUserSubscription != null) {
        servicesUserSubscription!.cancel();
        servicesUserSubscription = null;
      }  
    }
  }

  static Future triggerActions(List listActionName) async {
    await _channel.invokeMethod('triggerActions', [listActionName]);
  }

  static Future triggerActionsWithProperties(List listActionName, List listProperties) async {
    await _channel.invokeMethod('triggerActionsWithProperties', [listActionName, listProperties]);
  }

  static Future triggerActionsWithCompletion(List listActionName, List listProperties) async {
    await _channel.invokeMethod('triggerActionsWithCompletion', [listActionName, listProperties]);
  }

  static Future triggerActionsWithTags(List tags) async {
    await _channel.invokeMethod('triggerActionsWithTags', [tags]);
  }

  static Future triggerAction(String actionName) async {
    await _channel.invokeMethod('triggerAction', [actionName]);
  }

  static Future triggerActionWithProperties(String actionName, Map properties) async {
    await _channel.invokeMethod('triggerActionWithProperties', [actionName, properties]);
  }

  static Future triggerActionWithCompletion(String actionName) async {
    await _channel.invokeMethod('triggerActionWithCompletion', [actionName]);
  }

  static Future triggerActionWithPropertiesAndCompletion(String actionName, Map properties) async {
    await _channel.invokeMethod('triggerActionWithPropertiesAndCompletion', [actionName, properties]);
  }

  static Future triggerActionWithTag(Map tag) async {
    await _channel.invokeMethod('triggerActionWithTag', [tag]);
  }

  static Future showBadge() async {
    await _channel.invokeMethod('showBadge');
  }

  static Future hideBadge() async {
    await _channel.invokeMethod('hideBadge');
  }

  static Future setOnBadgeClickListener(void onData(dynamic event)?) async {
    if(onData != null) {
      if(onBadgeClickSubscription != null) {
        onBadgeClickSubscription!.cancel();
        onBadgeClickSubscription = null;
      }
      onBadgeClickSubscription = _onBadgeClickEventChannel.receiveBroadcastStream().listen(onData);
    }
    else {
      if(onBadgeClickSubscription != null) {
        onBadgeClickSubscription!.cancel();
        onBadgeClickSubscription = null;
      }
    }
  }

  static Future<bool> isOpenBigBadge() async {
    return await _channel.invokeMethod('isOpenBigBadge');
  }

  static Future openBigBadge() async {
    await _channel.invokeMethod('openBigBadge');
  }

  static Future openBigBadgeWithController(int controller) async {
    await _channel.invokeMethod('openBigBadgeWithController', [controller]);
  }

  static Future closeBigBadge() async {
    await _channel.invokeMethod('closeBigBadge');
  }

  static Future showOnboarding() async {
    await _channel.invokeMethod('showOnboarding');
  }

  static Future showOnboardingConnected(bool connected) async {
    await _channel.invokeMethod('showOnboardingConnected', [connected]);
  }

  static Future setBaseUrl(String baseUrl) async {
    await _channel.invokeMethod('setBaseUrl', [baseUrl]);
  }

  static Future<String> getBaseUrl() async {
    return await _channel.invokeMethod('getBaseUrl');
  }

  static Future setDebugMode(bool debugMode) async {
    await _channel.invokeMethod('setDebugMode', [debugMode]);
  }

  static Future<bool> isDebugMode() async {
    return await _channel.invokeMethod('isDebugMode');
  }

  static Future setDeviceStatus(int deviceStatus) async {
    await _channel.invokeMethod('setDeviceStatus', [deviceStatus]);
  }

  static Future setDeviceOptIn(int deviceOptIn) async {
    await _channel.invokeMethod('setDeviceOptIn', [deviceOptIn]);
  }

  static Future<Map> user() async {
    return await _channel.invokeMethod('user');
  }

  static Future<Map> device() async {
    return await _channel.invokeMethod('device');
  }

  static Future setBadgeEnabled(bool enabled) async {
    await _channel.invokeMethod('setBadgeEnabled', [enabled]);
  }

  static Future setInterstitialEnabled(bool enabled) async {
    await _channel.invokeMethod('setInterstitialEnabled', [enabled]);
  }

  static Future setNotificationFullscreenEnabled(bool enabled) async {
    await _channel.invokeMethod('setNotificationFullscreenEnabled', [enabled]);
  }
}
